interface EncoderConfig {
    width: number;
    height: number;
    frameRate: number;
    videoBitrate: number;
    audioBitrate: number;
    /**
     * Controls bitrate distribution for AAC. "constant" produces constant
     * bitrate (CBR) output while "variable" enables variable bitrate (VBR).
     * Not all browsers respect this setting. Chrome 119+ improves CBR support.
     */
    audioBitrateMode?: "constant" | "variable";
    sampleRate: number;
    channels: number;
    container?: "mp4" | "webm";
    codec?: {
        video?: "avc" | "hevc" | "vp9" | "vp8" | "av1";
        audio?: "aac" | "opus";
    };
    /**
     * Optional codec string overrides passed directly to the encoders.
     * For example: `{ video: 'avc1.640028', audio: 'mp4a.40.2' }`.
     */
    codecString?: {
        video?: string;
        audio?: string;
    };
    latencyMode?: "quality" | "realtime";
    /** Preference for hardware or software encoding. */
    hardwareAcceleration?: "prefer-hardware" | "prefer-software" | "no-preference";
    /** Drop new video frames when the number of queued frames exceeds `maxQueueDepth`. */
    dropFrames?: boolean;
    /** Maximum number of queued video frames before dropping. Defaults to `Infinity`. */
    maxQueueDepth?: number;
    /** Total frames for progress calculation if known in advance. */
    totalFrames?: number;
    /** Force a key frame every N video frames. */
    keyFrameInterval?: number;
    /**
     * How to handle the first timestamp of a track.
     * 'offset': Offsets all timestamps so the first one is 0.
     * 'strict': Requires the first timestamp to be 0 (default).
     */
    firstTimestampBehavior?: "offset" | "strict";
    /** Additional VideoEncoder configuration overrides. */
    videoEncoderConfig?: Partial<VideoEncoderConfig>;
    /** Additional AudioEncoder configuration overrides. */
    audioEncoderConfig?: Partial<AudioEncoderConfig>;
}
interface DetailedProgressInfo {
    /** 処理済みフレーム数 */
    processedFrames: number;
    /** 総フレーム数（不明の場合はundefined） */
    totalFrames?: number;
    /** 現在の処理ステージ */
    stage: ProcessingStage;
    /** 処理開始からの経過時間（ミリ秒） */
    elapsedTimeMs: number;
    /** 推定残り時間（ミリ秒、不明の場合はundefined） */
    estimatedRemainingMs?: number;
    /** フレーム/秒の処理速度 */
    processingFps: number;
    /** 平均処理速度（フレーム/秒） */
    averageProcessingFps: number;
    /** ドロップされたフレーム数 */
    droppedFrames: number;
    /** ビデオエンコーダーのキュー深度 */
    videoQueueSize: number;
    /** オーディオエンコーダーのキュー深度 */
    audioQueueSize: number;
    /** 処理済みデータサイズ（バイト） */
    processedDataSize?: number;
}
type ProgressCallback = (processedFrames: number, totalFrames?: number) => void;
type DetailedProgressCallback = (progress: DetailedProgressInfo) => void;
declare enum EncoderState {
    Idle = "idle",
    Initializing = "initializing",
    Encoding = "encoding",
    Finalizing = "finalizing",
    Disposed = "disposed",
    Error = "error"
}
declare enum ProcessingStage {
    Initializing = "initializing",
    VideoEncoding = "video-encoding",
    AudioEncoding = "audio-encoding",
    Muxing = "muxing",
    Finalizing = "finalizing"
}
declare enum EncoderErrorType {
    NotSupported = "not-supported",
    InitializationFailed = "initialization-failed",
    ConfigurationError = "configuration-error",
    EncodingFailed = "encoding-failed",// Generic encoding error
    VideoEncodingError = "video-encoding-error",// Specific video encoding error
    AudioEncodingError = "audio-encoding-error",// Specific audio encoding error
    MuxingFailed = "muxing-failed",
    Cancelled = "cancelled",
    Timeout = "timeout",
    InternalError = "internal-error",
    WorkerError = "worker-error",
    ValidationError = "validation-error"
}
declare class WebCodecsEncoderError extends Error {
    type: EncoderErrorType;
    cause?: unknown;
    constructor(type: EncoderErrorType, message: string, cause?: unknown);
}

type RealtimeDataCallback = (chunk: Uint8Array, offset?: number, isHeader?: boolean, container?: "mp4" | "webm") => void;
interface WebCodecsEncoderInitializeOptions {
    onProgress?: (processedFrames: number, totalFrames?: number) => void;
    onDetailedProgress?: DetailedProgressCallback;
    totalFrames?: number;
    onError?: (error: WebCodecsEncoderError) => void;
    onData?: RealtimeDataCallback;
    worker?: Worker;
    workerScriptUrl?: string | URL;
    useAudioWorklet?: boolean;
}
declare class WebCodecsEncoder {
    private config;
    private worker;
    private totalFrames;
    private processedFramesInternal;
    private submittedFramesInternal;
    private droppedFramesInternal;
    private videoQueueSizeInternal;
    private audioQueueSizeInternal;
    private actualVideoCodec;
    private actualAudioCodec;
    private currentState;
    private currentStage;
    private processingStartTime;
    private lastProgressTime;
    private lastProcessedFrames;
    private processedDataSize;
    private processingFpsHistory;
    private readonly maxFpsHistoryLength;
    private onInitialized;
    private onInitializeError;
    private onFinalizedPromise;
    private onProgressCallback;
    private onDetailedProgressCallback;
    private onErrorCallback;
    private onDataCallback;
    private isCancelled;
    private nextVideoTimestamp;
    private nextAudioTimestamp;
    private audioContext;
    private audioWorkletNode;
    private cancelTimeoutId;
    private configuredWorker?;
    private configuredWorkerScriptUrl?;
    private configuredUseAudioWorklet?;
    constructor(config?: EncoderConfig);
    static isSupported(): boolean;
    /**
     * メソッドチェーン用のファクトリーメソッド
     */
    static create(): WebCodecsEncoder;
    /**
     * 現在の状態を取得
     */
    getState(): EncoderState;
    /**
     * 現在の処理ステージを取得
     */
    getCurrentStage(): ProcessingStage;
    /**
     * 指定された状態でのAPI呼び出しが許可されているかチェック
     */
    private checkStateForOperation;
    /**
     * 状態を変更
     */
    private setState;
    /**
     * 処理ステージを変更
     */
    private setStage;
    /**
     * エンコーダーの設定を行う（メソッドチェーン用）
     */
    configure(config: EncoderConfig): this;
    /**
     * プログレスコールバックを設定（メソッドチェーン用）
     */
    onProgress(callback: (processedFrames: number, totalFrames?: number) => void): this;
    /**
     * 詳細プログレスコールバックを設定（メソッドチェーン用）
     */
    onDetailedProgress(callback: DetailedProgressCallback): this;
    /**
     * エラーコールバックを設定（メソッドチェーン用）
     */
    onError(callback: (error: WebCodecsEncoderError) => void): this;
    /**
     * リアルタイムデータコールバックを設定（メソッドチェーン用）
     */
    onData(callback: RealtimeDataCallback): this;
    /**
     * 総フレーム数を設定（メソッドチェーン用）
     */
    withTotalFrames(totalFrames: number): this;
    /**
     * ワーカーを設定（メソッドチェーン用）
     */
    withWorker(worker: Worker): this;
    /**
     * ワーカースクリプトURLを設定（メソッドチェーン用）
     */
    withWorkerScript(url: string | URL): this;
    /**
     * AudioWorkletの使用を設定（メソッドチェーン用）
     */
    withAudioWorklet(useAudioWorklet?: boolean): this;
    /**
     * エンコーダーを開始（メソッドチェーン用のinitialize）
     */
    start(): Promise<this>;
    /**
     * エンコーダーを終了（メソッドチェーン用のfinalize）
     */
    finish(): Promise<Uint8Array | null>;
    /**
     * 詳細プログレス情報を計算
     */
    private calculateDetailedProgress;
    initialize(options?: WebCodecsEncoderInitializeOptions): Promise<void>;
    private static findWorkerScript;
    private static isDevEnvironment;
    private static createInlineWorker;
    private findAudioWorkletScript;
    private handleWorkerMessage;
    private handleError;
    addVideoFrame(frameSource: VideoFrame, timestampOverride?: number): Promise<void>;
    addCanvasFrame(canvas: HTMLCanvasElement | OffscreenCanvas): Promise<void>;
    addAudioBuffer(audioBuffer: AudioBuffer): Promise<void>;
    addAudioData(audioData: AudioData): Promise<void>;
    finalize(): Promise<Uint8Array | null>;
    cancel(): void;
    private clearCancelTimeout;
    private cleanupWorker;
    private cleanupWorkerOnError;
    private setupAudioWorklet;
    getActualVideoCodec(): string | null;
    getActualAudioCodec(): string | null;
    getVideoQueueSize(): number;
    getAudioQueueSize(): number;
    getAudioWorkletNode(): AudioWorkletNode | null;
}

declare class MediaStreamRecorder {
    private config;
    private encoder;
    private videoReader?;
    private audioReader?;
    private videoTrack?;
    private audioTrack?;
    private audioSource?;
    private recording;
    private onErrorCallback?;
    constructor(config: EncoderConfig & {
        firstTimestampBehavior?: "offset" | "strict";
    });
    static isSupported(): boolean;
    startRecording(stream: MediaStream, options?: WebCodecsEncoderInitializeOptions): Promise<void>;
    private processVideo;
    private processAudio;
    stopRecording(): Promise<Uint8Array | null>;
    cancel(): void;
    getActualVideoCodec(): string | null;
    getActualAudioCodec(): string | null;
}

export { type EncoderConfig, EncoderErrorType, MediaStreamRecorder, type ProgressCallback, type RealtimeDataCallback, WebCodecsEncoder, WebCodecsEncoderError, type WebCodecsEncoderInitializeOptions };
